"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.changeUserStatus = exports.findByName = exports.getOneUser = exports.getUsers = exports.updateNewMessages = exports.getNewMessages = exports.logoutUser = exports.loginUser = exports.resendRegisterVerification = exports.confirmRegistration = exports.signupUser = void 0;
const bcrypt_1 = __importDefault(require("bcrypt"));
const mongoose_1 = require("mongoose");
const config_1 = require("../config/config");
const User_1 = require("../models/User");
const user_dto_1 = require("../types/user.dto");
const logs_1 = require("../utils/logs");
const mailer_1 = require("../utils/mailer");
const mailer_utils_1 = require("../utils/mailer.utils");
const signupUser = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { name, email, password, image } = req.body;
        if (!name || !email || !password) {
            return res.status(400).json({
                invalid: "Please add all fields",
            });
        }
        const userExist = yield User_1.User.findOne({ email });
        if (userExist) {
            return res.status(400).json({
                invalid: "User already exists",
            });
        }
        //hash password
        const salt = yield bcrypt_1.default.genSalt(10);
        const hashedPassword = yield bcrypt_1.default.hash(password, salt);
        //create user
        const user = yield new User_1.User({
            name,
            email,
            password: hashedPassword,
            image: image ? image : "",
            status: user_dto_1.UserStatus.offline,
            confirmed: false,
        });
        yield user.save();
        const confirmedLink = `${config_1.config.domainAddress}/users/confirm/${user._id}`;
        (0, mailer_1.sendMail)(email, mailer_utils_1.EmailSubject.register, mailer_utils_1.EmailView.register, confirmedLink);
        res.status(200).json({
            message: "Check your email and activate your account.",
        });
    }
    catch (e) {
        res.status(400).json({
            invalid: "Error, try later.",
        });
    }
});
exports.signupUser = signupUser;
const confirmRegistration = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const userId = String(req.params.id);
    if (!userId) {
        return res.status(400).render("partials/failed", {
            message: "Invalid id",
        });
    }
    if (!(0, mongoose_1.isValidObjectId)(userId)) {
        return res.status(400).render("partials/failed", {
            message: "Invalid id",
        });
    }
    const newUser = yield User_1.User.findOne({
        _id: userId,
    });
    if (!newUser) {
        return res.status(400).render("partials/failed", {
            message: "Invalid id",
        });
    }
    if (newUser.confirmed) {
        return res.status(400).render("partials/failed", {
            message: "Email has been confirmed before.",
        });
    }
    newUser.confirmed = true;
    yield newUser.save();
    res.status(200).render("partials/confirmed");
});
exports.confirmRegistration = confirmRegistration;
const resendRegisterVerification = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const email = String(req.body.email);
    if (!email) {
        res.status(400).json({
            invalid: "No email given.",
        });
        return;
    }
    const user = yield User_1.User.findOne({ email });
    if (!user) {
        res.status(400).json({
            invalid: "There is no user with given email.",
        });
        return;
    }
    if (user.disabled) {
        res.status(400).json({
            invalid: "The email address has been disabled by Administrator",
        });
        return;
    }
    if (user.confirmed) {
        res.status(400).json({
            invalid: "This email has already been confirmed.",
        });
        return;
    }
    const confirmedLink = `${config_1.config.domainAddress}/users/confirm/${user._id}`;
    (0, mailer_1.sendMail)(email, mailer_utils_1.EmailSubject.register, mailer_utils_1.EmailView.register, confirmedLink);
    res.status(200).json({
        message: "Check your email, we send you activation link.",
    });
});
exports.resendRegisterVerification = resendRegisterVerification;
const loginUser = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { email, password } = req.body;
        if (!email || !password) {
            return res.status(400).json({
                invalid: "Invalid credentials",
            });
        }
        const user = yield User_1.User.findOne({ email });
        if (!user) {
            return res.status(400).json({
                invalid: "User not found. Go to register page.",
            });
        }
        if (user.disabled) {
            res.status(400).json({
                invalid: "The email address has been disabled by Administrator",
            });
            return;
        }
        const passMatch = yield bcrypt_1.default.compare(password, user.password);
        if (!passMatch) {
            return res.status(400).json({
                invalid: "Invalid email or password",
            });
        }
        if (!user.confirmed) {
            return res.status(401).json({
                unconfirmed: "This account is not confirmed. Check your email and confirm registration.",
            });
        }
        const token = (0, logs_1.generateToken)(user._id);
        const hashedData = (0, logs_1.generateHashedData)(user._id, user.name, user.email, user.image, user.newMessages);
        user.token = token;
        user.status = user_dto_1.UserStatus.online;
        yield user.save();
        res.status(200).json({
            data: hashedData,
            token: user.token,
        });
    }
    catch (e) {
        res.status(400).json({
            invalid: "Error, try later.",
        });
    }
});
exports.loginUser = loginUser;
const logoutUser = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const user = req.user;
    if (!user) {
        return res.status(400).json({
            invalid: "Invalid user id",
        });
    }
    user.status = user_dto_1.UserStatus.offline;
    // user.newMessages = newMessages;
    yield user.save();
    res.status(200).json({
        message: "Logout successfully",
    });
});
exports.logoutUser = logoutUser;
const getNewMessages = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const user = req.user;
    if (!user) {
        return res.status(400).json({
            invalid: "Invalid user id",
        });
    }
    res.status(200).json({
        // messages: user.newMessages ?? {},
        newMessages: user.newMessages,
    });
});
exports.getNewMessages = getNewMessages;
const updateNewMessages = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    var _a;
    const user = req.user;
    const newMessages = req.body;
    user.newMessages = newMessages !== null && newMessages !== void 0 ? newMessages : {};
    user.markModified("newMessages");
    yield user.save();
    res.status(200).json({
        messages: (_a = user.newMessages) !== null && _a !== void 0 ? _a : {},
    });
});
exports.updateNewMessages = updateNewMessages;
const getUsers = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const members = yield User_1.User.find();
    res.status(200).json({
        users: exportedMembersData(members),
    });
});
exports.getUsers = getUsers;
const getOneUser = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const userId = req.params.id;
    // const user = await User.findById(userId);
    const user = yield User_1.User.findOne({ _id: userId });
    if (!user) {
        return res.status(404).json({
            error: "no user",
        });
    }
    res.status(200).json({ user: exportedMembersData([user]) });
});
exports.getOneUser = getOneUser;
const findByName = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const name = String(req.body.name).toLocaleLowerCase();
    const userId = req.user._id.toString();
    const results = yield User_1.User.find()
        // .and([{ name: { $regex: name } }, { _id: { $ne: userId } }])
        .and([{ name: { $regex: name } }])
        .limit(5);
    if (results.length === 0) {
        return res.status(404).json({
            error: "no results",
        });
    }
    res.status(200).json({
        users: exportedMembersData(results),
    });
});
exports.findByName = findByName;
const changeUserStatus = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const user = req.user;
    const { status } = req.body;
    console.log("body status: ", status);
    user.status =
        status === user_dto_1.UserStatus.online ? user_dto_1.UserStatus.online : user_dto_1.UserStatus.offline;
    // await user.save();
    try {
        yield user.save();
    }
    catch (error) {
    }
    finally {
        console.log("działa zmiana statusu: ", user.name, " - ", user.status);
    }
    res.status(200).json({
        status: user.status,
    });
});
exports.changeUserStatus = changeUserStatus;
const exportedMembersData = (members) => {
    return members.map(({ _id, email, name, image, status }) => ({
        _id,
        name,
        image,
        status,
    }));
};
//# sourceMappingURL=userController.js.map